@@ -22,36 +22,52 @@ from ..mesonlib import version_compare
 
 import typing as T
 from pathlib import Path
+from functools import lru_cache
 import re
-import os
 import json
 import textwrap
 
 class CMakeTraceLine:
-    def __init__(self, file, line, func, args):
-        self.file = file
+    def __init__(self, file_str: str, line: int, func: str, args: T.List[str]) -> None:
+        self.file = CMakeTraceLine._to_path(file_str)
         self.line = line
         self.func = func.lower()
         self.args = args
 
-    def __repr__(self):
+    @staticmethod
+    @lru_cache(maxsize=None)
+    def _to_path(file_str: str) -> Path:
+        return Path(file_str)
+
+    def __repr__(self) -> str:
         s = 'CMake TRACE: {0}:{1} {2}({3})'
         return s.format(self.file, self.line, self.func, self.args)
 
+class CMakeCacheEntry(T.NamedTuple):
+    value: T.List[str]
+    type: str
+
 class CMakeTarget:
-    def __init__(self, name, target_type, properties=None, imported: bool = False, tline: T.Optional[CMakeTraceLine] = None):
+    def __init__(
+                self,
+                name:        str,
+                target_type: str,
+                properties:  T.Optional[T.Dict[str, T.List[str]]] = None,
+                imported:    bool                                 = False,
+                tline:       T.Optional[CMakeTraceLine]           = None
+            ):
         if properties is None:
             properties = {}
-        self.name = name
-        self.type = target_type
-        self.properties = properties
-        self.imported = imported
-        self.tline = tline
-        self.depends = []
-        self.current_bin_dir = None
-        self.current_src_dir = None
+        self.name            = name
+        self.type            = target_type
+        self.properties      = properties
+        self.imported        = imported
+        self.tline           = tline
+        self.depends         = []      # type: T.List[str]
+        self.current_bin_dir = None    # type: T.Optional[Path]
+        self.current_src_dir = None    # type: T.Optional[Path]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         s = 'CMake TARGET:\n  -- name:      {}\n  -- type:      {}\n  -- imported:  {}\n  -- properties: {{\n{}     }}\n  -- tline: {}'
         propSTR = ''
         for i in self.properties:
@@ -67,16 +83,18 @@ class CMakeTarget:
             assert all([';' not in x for x in self.properties[key]])
 
 class CMakeGeneratorTarget(CMakeTarget):
-    def __init__(self, name):
+    def __init__(self, name: str) -> None:
         super().__init__(name, 'CUSTOM', {})
-        self.outputs = []        # type: T.List[str]
+        self.outputs = []        # type: T.List[Path]
         self.command = []        # type: T.List[T.List[str]]
-        self.working_dir = None  # type: T.Optional[str]
+        self.working_dir = None  # type: T.Optional[Path]
 
 class CMakeTraceParser:
-    def __init__(self, cmake_version: str, build_dir: str, permissive: bool = True):
-        self.vars = {}     # type: T.Dict[str, T.List[str]]
-        self.targets = {}  # type: T.Dict[str, CMakeTarget]
+    def __init__(self, cmake_version: str, build_dir: Path, permissive: bool = True) -> None:
+        self.vars:                      T.Dict[str, T.List[str]]     = {}
+        self.vars_by_file: T.Dict[Path, T.Dict[str, T.List[str]]]    = {}
+        self.targets:                   T.Dict[str, CMakeTarget]     = {}
+        self.cache:                     T.Dict[str, CMakeCacheEntry] = {}
 
         self.explicit_headers = set()  # type: T.Set[Path]
 
@@ -86,7 +104,7 @@ class CMakeTraceParser:
         self.permissive = permissive  # type: bool
         self.cmake_version = cmake_version  # type: str
         self.trace_file = 'cmake_trace.txt'
-        self.trace_file_path = Path(build_dir) / self.trace_file
+        self.trace_file_path = build_dir / self.trace_file
         self.trace_format = 'json-v1' if version_compare(cmake_version, '>=3.17') else 'human'
 
         # State for delayed command execution. Delayed command execution is realised
@@ -117,7 +135,8 @@ class CMakeTraceParser:
             # meaning here in the trace parser.
             'meson_ps_execute_delayed_calls': self._meson_ps_execute_delayed_calls,
             'meson_ps_reload_vars': self._meson_ps_reload_vars,
-        }
+            'meson_ps_disabled_function': self._meson_ps_disabled_function,
+        }  # type: T.Dict[str, T.Callable[[CMakeTraceLine], None]]
 
     def trace_args(self) -> T.List[str]:
         arg_map = {
@@ -127,7 +146,7 @@ class CMakeTraceParser:
 
         base_args = ['--no-warn-unused-cli']
         if not self.requires_stderr():
-            base_args += ['--trace-redirect={}'.format(self.trace_file)]
+            base_args += [f'--trace-redirect={self.trace_file}']
 
         return arg_map[self.trace_format] + base_args
 
@@ -138,8 +157,8 @@ class CMakeTraceParser:
         # First load the trace (if required)
         if not self.requires_stderr():
             if not self.trace_file_path.exists and not self.trace_file_path.is_file():
-                raise CMakeException('CMake: Trace file "{}" not found'.format(str(self.trace_file_path)))
-            trace = self.trace_file_path.read_text(errors='ignore')
+                raise CMakeException(f'CMake: Trace file "{self.trace_file_path!s}" not found')
+            trace = self.trace_file_path.read_text(errors='ignore', encoding='utf-8')
         if not trace:
             raise CMakeException('CMake: The CMake trace was not provided or is empty')
 
@@ -150,7 +169,7 @@ class CMakeTraceParser:
         elif self.trace_format == 'json-v1':
             lexer1 = self._lex_trace_json(trace)
         else:
-            raise CMakeException('CMake: Internal error: Invalid trace format {}. Expected [human, json-v1]'.format(self.trace_format))
+            raise CMakeException(f'CMake: Internal error: Invalid trace format {self.trace_format}. Expected [human, json-v1]')
 
         # Primary pass -- parse everything
         for l in lexer1:
@@ -161,7 +180,7 @@ class CMakeTraceParser:
 
             # "Execute" the CMake function if supported
             fn = self.functions.get(l.func, None)
-            if(fn):
+            if fn:
                 fn(l)
 
         # Postprocess
@@ -206,9 +225,9 @@ class CMakeTraceParser:
         # Generate an exception if the parser is not in permissive mode
 
         if self.permissive:
-            mlog.debug('CMake trace warning: {}() {}\n{}'.format(function, error, tline))
+            mlog.debug(f'CMake trace warning: {function}() {error}\n{tline}')
             return None
-        raise CMakeException('CMake: {}() {}\n{}'.format(function, error, tline))
+        raise CMakeException(f'CMake: {function}() {error}\n{tline}')
 
     def _cmake_set(self, tline: CMakeTraceLine) -> None:
         """Handler for the CMake set() function in all variaties.
@@ -227,6 +246,14 @@ class CMakeTraceParser:
         """
         # DOC: https://cmake.org/cmake/help/latest/command/set.html
 
+        cache_type  = None
+        cache_force = 'FORCE' in tline.args
+        try:
+            cache_idx  = tline.args.index('CACHE')
+            cache_type = tline.args[cache_idx + 1]
+        except (ValueError, IndexError):
+            pass
+
         # 1st remove PARENT_SCOPE and CACHE from args
         args = []
         for i in tline.args:
@@ -249,14 +276,21 @@ class CMakeTraceParser:
         identifier = args.pop(0)
         value = ' '.join(args)
 
+        # Write to the CMake cache instead
+        if cache_type:
+            # Honor how the CMake FORCE parameter works
+            if identifier not in self.cache or cache_force:
+                self.cache[identifier] = CMakeCacheEntry(value.split(';'), cache_type)
+
         if not value:
             # Same as unset
             if identifier in self.vars:
                 del self.vars[identifier]
         else:
             self.vars[identifier] = value.split(';')
+            self.vars_by_file.setdefault(tline.file, {})[identifier] = value.split(';')
 
-    def _cmake_unset(self, tline: CMakeTraceLine):
+    def _cmake_unset(self, tline: CMakeTraceLine) -> None:
         # DOC: https://cmake.org/cmake/help/latest/command/unset.html
         if len(tline.args) < 1:
             return self._gen_exception('unset', 'requires at least one argument', tline)
@@ -264,7 +298,7 @@ class CMakeTraceParser:
         if tline.args[0] in self.vars:
             del self.vars[tline.args[0]]
 
-    def _cmake_add_executable(self, tline: CMakeTraceLine):
+    def _cmake_add_executable(self, tline: CMakeTraceLine) -> None:
         # DOC: https://cmake.org/cmake/help/latest/command/add_executable.html
         args = list(tline.args) # Make a working copy
 
@@ -280,7 +314,7 @@ class CMakeTraceParser:
 
         self.targets[args[0]] = CMakeTarget(args[0], 'EXECUTABLE', {}, tline=tline, imported=is_imported)
 
-    def _cmake_add_library(self, tline: CMakeTraceLine):
+    def _cmake_add_library(self, tline: CMakeTraceLine) -> None:
         # DOC: https://cmake.org/cmake/help/latest/command/add_library.html
         args = list(tline.args) # Make a working copy
 
@@ -314,9 +348,9 @@ class CMakeTraceParser:
         else:
             self.targets[args[0]] = CMakeTarget(args[0], 'NORMAL', {}, tline=tline)
 
-    def _cmake_add_custom_command(self, tline: CMakeTraceLine, name=None):
+    def _cmake_add_custom_command(self, tline: CMakeTraceLine, name: T.Optional[str] = None) -> None:
         # DOC: https://cmake.org/cmake/help/latest/command/add_custom_command.html
-        args = self._flatten_args(list(tline.args))  # Commands can be passed as ';' seperated lists
+        args = self._flatten_args(list(tline.args))  # Commands can be passed as ';' separated lists
 
         if not args:
             return self._gen_exception('add_custom_command', 'requires at least 1 argument', tline)
@@ -332,7 +366,7 @@ class CMakeTraceParser:
         target = CMakeGeneratorTarget(name)
 
         def handle_output(key: str, target: CMakeGeneratorTarget) -> None:
-            target.outputs += [key]
+            target.outputs += [Path(key)]
 
         def handle_command(key: str, target: CMakeGeneratorTarget) -> None:
             if key == 'ARGS':
@@ -342,12 +376,14 @@ class CMakeTraceParser:
         def handle_depends(key: str, target: CMakeGeneratorTarget) -> None:
             target.depends += [key]
 
+        working_dir = None
         def handle_working_dir(key: str, target: CMakeGeneratorTarget) -> None:
-            if target.working_dir is None:
-                target.working_dir = key
+            nonlocal working_dir
+            if working_dir is None:
+                working_dir = key
             else:
-                target.working_dir += ' '
-                target.working_dir += key
+                working_dir += ' '
+                working_dir += key
 
         fn = None
 
@@ -369,9 +405,13 @@ class CMakeTraceParser:
             if fn is not None:
                 fn(i, target)
 
-        target.current_bin_dir = self.var_to_str('MESON_PS_CMAKE_CURRENT_BINARY_DIR')
-        target.current_src_dir = self.var_to_str('MESON_PS_CMAKE_CURRENT_SOURCE_DIR')
-        target.outputs = self._guess_files(target.outputs)
+        cbinary_dir = self.var_to_str('MESON_PS_CMAKE_CURRENT_BINARY_DIR')
+        csource_dir = self.var_to_str('MESON_PS_CMAKE_CURRENT_SOURCE_DIR')
+
+        target.working_dir     = Path(working_dir) if working_dir else None
+        target.current_bin_dir = Path(cbinary_dir) if cbinary_dir else None
+        target.current_src_dir = Path(csource_dir) if csource_dir else None
+        target.outputs = [Path(x) for x in self._guess_files([str(y) for y in target.outputs])]
         target.depends = self._guess_files(target.depends)
         target.command = [self._guess_files(x) for x in target.command]
 
@@ -379,7 +419,7 @@ class CMakeTraceParser:
         if name:
             self.targets[name] = target
 
-    def _cmake_add_custom_target(self, tline: CMakeTraceLine):
+    def _cmake_add_custom_target(self, tline: CMakeTraceLine) -> None:
         # DOC: https://cmake.org/cmake/help/latest/command/add_custom_target.html
         # We only the first parameter (the target name) is interesting
         if len(tline.args) < 1:
@@ -424,17 +464,18 @@ class CMakeTraceParser:
         if not value:
             return
 
-        def do_target(tgt: str) -> None:
-            if i not in self.targets:
-                return self._gen_exception('set_property', 'TARGET {} not found'.format(i), tline)
+        def do_target(t: str) -> None:
+            if t not in self.targets:
+                return self._gen_exception('set_property', f'TARGET {t} not found', tline)
 
-            if identifier not in self.targets[i].properties:
-                self.targets[i].properties[identifier] = []
+            tgt = self.targets[t]
+            if identifier not in tgt.properties:
+                tgt.properties[identifier] = []
 
             if append:
-                self.targets[i].properties[identifier] += value
+                tgt.properties[identifier] += value
             else:
-                self.targets[i].properties[identifier] = value
+                tgt.properties[identifier] = value
 
         def do_source(src: str) -> None:
             if identifier != 'HEADER_FILE_ONLY' or not self._str_to_bool(value):
@@ -494,7 +535,7 @@ class CMakeTraceParser:
         arglist = []  # type: T.List[T.Tuple[str, T.List[str]]]
         if self.trace_format == 'human':
             name = args.pop(0)
-            values = []
+            values = []  # type: T.List[str]
             prop_regex = re.compile(r'^[A-Z_]+$')
             for a in args:
                 if prop_regex.match(a):
@@ -512,7 +553,7 @@ class CMakeTraceParser:
         for name, value in arglist:
             for i in targets:
                 if i not in self.targets:
-                    return self._gen_exception('set_target_properties', 'TARGET {} not found'.format(i), tline)
+                    return self._gen_exception('set_target_properties', f'TARGET {i} not found', tline)
 
                 self.targets[i].properties[name] = value
 
@@ -550,7 +591,7 @@ class CMakeTraceParser:
         # DOC: https://cmake.org/cmake/help/latest/command/target_link_libraries.html
         self._parse_common_target_options('target_link_options', 'LINK_LIBRARIES', 'INTERFACE_LINK_LIBRARIES', tline)
 
-    def _parse_common_target_options(self, func: str, private_prop: str, interface_prop: str, tline: CMakeTraceLine, ignore: T.Optional[T.List[str]] = None, paths: bool = False):
+    def _parse_common_target_options(self, func: str, private_prop: str, interface_prop: str, tline: CMakeTraceLine, ignore: T.Optional[T.List[str]] = None, paths: bool = False) -> None:
         if ignore is None:
             ignore = ['BEFORE']
 
@@ -561,7 +602,7 @@ class CMakeTraceParser:
 
         target = args[0]
         if target not in self.targets:
-            return self._gen_exception(func, 'TARGET {} not found'.format(target), tline)
+            return self._gen_exception(func, f'TARGET {target} not found', tline)
 
         interface = []
         private = []
@@ -588,16 +629,16 @@ class CMakeTraceParser:
         interface = [x for x in interface if x]
         private = [x for x in private if x]
 
-        for i in [(private_prop, private), (interface_prop, interface)]:
-            if not i[0] in self.targets[target].properties:
-                self.targets[target].properties[i[0]] = []
+        for j in [(private_prop, private), (interface_prop, interface)]:
+            if not j[0] in self.targets[target].properties:
+                self.targets[target].properties[j[0]] = []
 
-            self.targets[target].properties[i[0]] += i[1]
+            self.targets[target].properties[j[0]] += j[1]
 
     def _meson_ps_execute_delayed_calls(self, tline: CMakeTraceLine) -> None:
         for l in self.stored_commands:
             fn = self.functions.get(l.func, None)
-            if(fn):
+            if fn:
                 fn(l)
 
         # clear the stored commands
@@ -606,7 +647,14 @@ class CMakeTraceParser:
     def _meson_ps_reload_vars(self, tline: CMakeTraceLine) -> None:
         self.delayed_commands = self.get_cmake_var('MESON_PS_DELAYED_CALLS')
 
-    def _lex_trace_human(self, trace):
+    def _meson_ps_disabled_function(self, tline: CMakeTraceLine) -> None:
+        args = list(tline.args)
+        if not args:
+            mlog.error('Invalid preload.cmake script! At least one argument to `meson_ps_disabled_function` is expected')
+            return
+        mlog.warning(f'The CMake function "{args[0]}" was disabled to avoid compatibility issues with Meson.')
+
+    def _lex_trace_human(self, trace: str) -> T.Generator[CMakeTraceLine, None, None]:
         # The trace format is: '<file>(<line>):  <func>(<args -- can contain \n> )\n'
         reg_tline = re.compile(r'\s*(.*\.(cmake|txt))\(([0-9]+)\):\s*(\w+)\(([\s\S]*?) ?\)\s*\n', re.MULTILINE)
         reg_other = re.compile(r'[^\n]*\n')
@@ -629,17 +677,23 @@ class CMakeTraceParser:
             func = mo_file_line.group(4)
             args = mo_file_line.group(5)
             args = parse_generator_expressions(args)
-            args = args.split(' ')
-            args = list(map(lambda x: x.strip(), args))
+            argl = args.split(' ')
+            argl = list(map(lambda x: x.strip(), argl))
 
-            yield CMakeTraceLine(file, line, func, args)
+            yield CMakeTraceLine(file, int(line), func, argl)
 
-    def _lex_trace_json(self, trace: str):
+    def _lex_trace_json(self, trace: str) -> T.Generator[CMakeTraceLine, None, None]:
         lines = trace.splitlines(keepends=False)
         lines.pop(0)  # The first line is the version
         for i in lines:
             data = json.loads(i)
+            assert isinstance(data['file'], str)
+            assert isinstance(data['line'], int)
+            assert isinstance(data['cmd'],  str)
+            assert isinstance(data['args'], list)
             args = data['args']
+            for j in args:
+                assert isinstance(j, str)
             args = [parse_generator_expressions(x) for x in args]
             yield CMakeTraceLine(data['file'], data['line'], data['cmd'], args)
 
@@ -668,7 +722,7 @@ class CMakeTraceParser:
             if curr_str is None:
                 curr_str = i
                 path_found = False
-            elif os.path.isfile(curr_str):
+            elif Path(curr_str).is_file():
                 # Abort concatenation if curr_str is an existing file
                 fixed_list += [curr_str]
                 curr_str = i
@@ -680,13 +734,13 @@ class CMakeTraceParser:
                 path_found = False
             elif reg_end.match(i):
                 # File detected
-                curr_str = '{} {}'.format(curr_str, i)
+                curr_str = f'{curr_str} {i}'
                 fixed_list += [curr_str]
                 curr_str = None
                 path_found = False
-            elif os.path.exists('{} {}'.format(curr_str, i)):
+            elif Path(f'{curr_str} {i}').exists():
                 # Path detected
-                curr_str = '{} {}'.format(curr_str, i)
+                curr_str = f'{curr_str} {i}'
                 path_found = True
             elif path_found:
                 # Add path to fixed_list after ensuring the whole path is in curr_str
@@ -694,7 +748,7 @@ class CMakeTraceParser:
                 curr_str = i
                 path_found = False
             else:
-                curr_str = '{} {}'.format(curr_str, i)
+                curr_str = f'{curr_str} {i}'
                 path_found = False
 
         if curr_str:
