@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
 import re
 import functools
 import typing as T
@@ -22,7 +21,8 @@ from .. import mlog
 from .. import mesonlib
 from ..environment import Environment
 
-from .base import DependencyException, ExternalDependency, PkgConfigDependency
+from .base import DependencyException, SystemDependency
+from .pkgconfig import PkgConfigDependency
 from .misc import threads_factory
 
 if T.TYPE_CHECKING:
@@ -240,7 +240,7 @@ class BoostLibraryFile():
     def fix_python_name(self, tags: T.List[str]) -> T.List[str]:
         # Handle the boost_python naming madeness.
         # See https://github.com/mesonbuild/meson/issues/4788 for some distro
-        # specific naming variantions.
+        # specific naming variations.
         other_tags = []  # type: T.List[str]
 
         # Split the current modname into the base name and the version
@@ -320,7 +320,7 @@ class BoostLibraryFile():
         elif vscrt in ['/MTd', '-MTd']:
             return (self.runtime_static or not self.static) and self.runtime_debug
 
-        mlog.warning(f'Boost: unknow vscrt tag {vscrt}. This may cause the compilation to fail. Please consider reporting this as a bug.', once=True)
+        mlog.warning(f'Boost: unknown vscrt tag {vscrt}. This may cause the compilation to fail. Please consider reporting this as a bug.', once=True)
         return True
 
     def get_compiler_args(self) -> T.List[str]:
@@ -340,7 +340,7 @@ class BoostLibraryFile():
     def get_link_args(self) -> T.List[str]:
         return [self.path.as_posix()]
 
-class BoostDependency(ExternalDependency):
+class BoostDependency(SystemDependency):
     def __init__(self, environment: Environment, kwargs: T.Dict[str, T.Any]) -> None:
         super().__init__('boost', environment, kwargs, language='cpp')
         buildtype = environment.coredata.get_option(mesonlib.OptionKey('buildtype'))
@@ -382,7 +382,7 @@ class BoostDependency(ExternalDependency):
         # Finally, look for paths from .pc files and from searching the filesystem
         self.detect_roots()
 
-    def check_and_set_roots(self, roots: T.List[Path]) -> None:
+    def check_and_set_roots(self, roots: T.List[Path], use_system: bool) -> None:
         roots = list(mesonlib.OrderedSet(roots))
         for j in roots:
             #   1. Look for the boost headers (boost/version.hpp)
@@ -394,7 +394,7 @@ class BoostDependency(ExternalDependency):
             if not inc_dirs:
                 continue
 
-            lib_dirs = self.detect_lib_dirs(j)
+            lib_dirs = self.detect_lib_dirs(j, use_system)
             self.is_found = self.run_check(inc_dirs, lib_dirs)
             if self.is_found:
                 self.boost_root = j
@@ -405,7 +405,7 @@ class BoostDependency(ExternalDependency):
 
         The machine file values are defaulted to the environment values.
         """
-        # XXX: if we had a TypedDict we woudn't need this
+        # XXX: if we had a TypedDict we wouldn't need this
         incdir = props.get('boost_includedir')
         assert incdir is None or isinstance(incdir, str)
         libdir = props.get('boost_librarydir')
@@ -429,14 +429,14 @@ class BoostDependency(ExternalDependency):
 
         rootdir = props.get('boost_root')
         # It shouldn't be possible to get here without something in boost_root
-        assert(rootdir)
+        assert rootdir
 
         raw_paths = mesonlib.stringlistify(rootdir)
         paths = [Path(x) for x in raw_paths]
         if paths and any([not x.is_absolute() for x in paths]):
             raise DependencyException('boost_root path given in machine file must be absolute')
 
-        self.check_and_set_roots(paths)
+        self.check_and_set_roots(paths, use_system=False)
 
     def run_check(self, inc_dirs: T.List[BoostIncludeDir], lib_dirs: T.List[Path]) -> bool:
         mlog.debug('  - potential library dirs: {}'.format([x.as_posix() for x in lib_dirs]))
@@ -532,17 +532,19 @@ class BoostDependency(ExternalDependency):
         candidates = [x for x in candidates if x.exists()]
         return [self._include_dir_from_version_header(x) for x in candidates]
 
-    def detect_lib_dirs(self, root: Path) -> T.List[Path]:
+    def detect_lib_dirs(self, root: Path, use_system: bool) -> T.List[Path]:
         # First check the system include paths. Only consider those within the
         # given root path
-        system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
-        system_dirs = [Path(x) for x in system_dirs_t]
-        system_dirs = [x.resolve() for x in system_dirs if x.exists()]
-        system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
-        system_dirs = list(mesonlib.OrderedSet(system_dirs))
 
-        if system_dirs:
-            return system_dirs
+        if use_system:
+            system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
+            system_dirs = [Path(x) for x in system_dirs_t]
+            system_dirs = [x.resolve() for x in system_dirs if x.exists()]
+            system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
+            system_dirs = list(mesonlib.OrderedSet(system_dirs))
+
+            if system_dirs:
+                return system_dirs
 
         # No system include paths were found --> fall back to manually looking
         # for library dirs in root
@@ -615,14 +617,15 @@ class BoostDependency(ExternalDependency):
         return libs
 
     def detect_libraries(self, libdir: Path) -> T.List[BoostLibraryFile]:
-        libs = []  # type: T.List[BoostLibraryFile]
+        libs = set()  # type: T.Set[BoostLibraryFile]
         for i in libdir.iterdir():
-            if not i.is_file() or i.is_symlink():
+            if not i.is_file():
                 continue
             if not any([i.name.startswith(x) for x in ['libboost_', 'boost_']]):
                 continue
 
-            libs += [BoostLibraryFile(i)]
+            libs.add(BoostLibraryFile(i.resolve()))
+
         return [x for x in libs if x.is_boost()]  # Filter out no boost libraries
 
     def detect_split_root(self, inc_dir: Path, lib_dir: Path) -> None:
@@ -679,11 +682,6 @@ class BoostDependency(ExternalDependency):
         else:
             tmp = []  # type: T.List[Path]
 
-            # Homebrew
-            brew_boost = Path('/usr/local/Cellar/boost')
-            if brew_boost.is_dir():
-                tmp += [x for x in brew_boost.iterdir()]
-
             # Add some default system paths
             tmp += [Path('/opt/local')]
             tmp += [Path('/usr/local/opt/boost')]
@@ -695,7 +693,7 @@ class BoostDependency(ExternalDependency):
             tmp = [x.resolve() for x in tmp]
             roots += tmp
 
-        self.check_and_set_roots(roots)
+        self.check_and_set_roots(roots, use_system=True)
 
     def log_details(self) -> str:
         res = ''
@@ -717,7 +715,7 @@ class BoostDependency(ExternalDependency):
         # also work, however, this is slower (since it the compiler has to be
         # invoked) and overkill since the layout of the header is always the same.
         assert hfile.exists()
-        raw = hfile.read_text()
+        raw = hfile.read_text(encoding='utf-8')
         m = re.search(r'#define\s+BOOST_VERSION\s+([0-9]+)', raw)
         if not m:
             mlog.debug(f'Failed to extract version information from {hfile}')
