@@ -466,15 +466,15 @@ class ZlibSystemDependency(SystemDependency):
         # I'm not sure this is entirely correct. What if we're cross compiling
         # from something to macOS?
         if ((m.is_darwin() and isinstance(self.clib_compiler, (AppleClangCCompiler, AppleClangCPPCompiler))) or
-                m.is_freebsd() or m.is_dragonflybsd()):
+                m.is_freebsd() or m.is_dragonflybsd() or m.is_android()):
             # No need to set includes,
             # on macos xcode/clang will do that for us.
             # on freebsd zlib.h is in /usr/include
 
             self.is_found = True
             self.link_args = ['-lz']
-        elif m.is_windows():
-            # Without a clib_compiler we can't find zlib, s just give up.
+        else:
+            # Without a clib_compiler we can't find zlib, so just give up.
             if self.clib_compiler is None:
                 self.is_found = False
                 return
@@ -492,9 +492,6 @@ class ZlibSystemDependency(SystemDependency):
                     break
             else:
                 return
-        else:
-            mlog.debug(f'Unsupported OS {m.system}')
-            return
 
         v, _ = self.clib_compiler.get_define('ZLIB_VERSION', '#include <zlib.h>', self.env, [], [self])
         self.version = v.strip('"')
