@@ -17,15 +17,33 @@ import typing as T
 
 from . import ExtensionModule, ModuleReturnValue
 from .. import mlog
-from ..build import BuildTarget, CustomTargetIndex, Executable, GeneratedList, InvalidArguments, IncludeDirs, CustomTarget
+from ..build import BothLibraries, BuildTarget, CustomTargetIndex, Executable, ExtractedObjects, GeneratedList, IncludeDirs, CustomTarget
 from ..dependencies import Dependency, ExternalLibrary
-from ..interpreter import ExecutableHolder, BuildTargetHolder, CustomTargetHolder, permitted_kwargs, noPosargs
-from ..interpreterbase import InterpreterException, permittedKwargs, FeatureNew, typed_pos_args
-from ..mesonlib import stringlistify, unholder, listify, typeslistify, File
+from ..interpreter.interpreter import TEST_KWARGS
+from ..interpreterbase import ContainerTypeInfo, InterpreterException, KwargInfo, FeatureNew, typed_kwargs, typed_pos_args, noPosargs
+from ..mesonlib import File
 
 if T.TYPE_CHECKING:
-    from ..interpreter import ModuleState, Interpreter
-    from ..dependencies import ExternalProgram
+    from . import ModuleState
+    from ..interpreter import Interpreter
+    from ..interpreter import kwargs as _kwargs
+    from ..interpreter.interpreter import SourceInputs, SourceOutputs
+    from ..programs import ExternalProgram
+
+    from typing_extensions import TypedDict
+
+    class FuncTest(_kwargs.BaseTest):
+
+        dependencies: T.List[T.Union[Dependency, ExternalLibrary]]
+        is_parallel: bool
+
+    class FuncBindgen(TypedDict):
+
+        args: T.List[str]
+        c_args: T.List[str]
+        include_directories: T.List[IncludeDirs]
+        input: T.List[SourceInputs]
+        output: str
 
 
 class RustModule(ExtensionModule):
@@ -36,10 +54,23 @@ class RustModule(ExtensionModule):
     def __init__(self, interpreter: 'Interpreter') -> None:
         super().__init__(interpreter)
         self._bindgen_bin: T.Optional['ExternalProgram'] = None
+        self.methods.update({
+            'test': self.test,
+            'bindgen': self.bindgen,
+        })
 
-    @permittedKwargs(permitted_kwargs['test'] | {'dependencies'} ^ {'protocol'})
-    @typed_pos_args('rust.test', str, BuildTargetHolder)
-    def test(self, state: 'ModuleState', args: T.Tuple[str, BuildTargetHolder], kwargs: T.Dict[str, T.Any]) -> ModuleReturnValue:
+    @typed_pos_args('rust.test', str, BuildTarget)
+    @typed_kwargs(
+        'rust.test',
+        *TEST_KWARGS,
+        KwargInfo('is_parallel', bool, default=False),
+        KwargInfo(
+            'dependencies',
+            ContainerTypeInfo(list, (Dependency, ExternalLibrary)),
+            listify=True,
+            default=[]),
+    )
+    def test(self, state: 'ModuleState', args: T.Tuple[str, BuildTarget], kwargs: 'FuncTest') -> ModuleReturnValue:
         """Generate a rust test target from a given rust target.
 
         Rust puts it's unitests inside it's main source files, unlike most
@@ -82,10 +113,10 @@ class RustModule(ExtensionModule):
         ```
         """
         name = args[0]
-        base_target: BuildTarget = unholder(args[1])
+        base_target: BuildTarget = args[1]
         if not base_target.uses_rust():
             raise InterpreterException('Second positional argument to rustmod.test() must be a rust based target')
-        extra_args = stringlistify(kwargs.get('args', []))
+        extra_args = kwargs['args']
 
         # Delete any arguments we don't want passed
         if '--test' in extra_args:
@@ -98,17 +129,17 @@ class RustModule(ExtensionModule):
             del extra_args[i + 1]
             del extra_args[i]
         for i, a in enumerate(extra_args):
-            if a.startswith('--format='):
+            if isinstance(a, str) and a.startswith('--format='):
                 del extra_args[i]
                 break
 
-        dependencies = unholder(listify(kwargs.get('dependencies', [])))
-        for d in dependencies:
-            if not isinstance(d, (Dependency, ExternalLibrary)):
-                raise InvalidArguments('dependencies must be a dependency or external library')
+        dependencies = [d for d in kwargs['dependencies']]
 
-        kwargs['args'] = extra_args + ['--test', '--format', 'pretty']
-        kwargs['protocol'] = 'rust'
+        # We need to cast here, as currently these don't have protocol in them, but test itself does.
+        tkwargs = T.cast('_kwargs.FuncTest', kwargs.copy())
+
+        tkwargs['args'] = extra_args + ['--test', '--format', 'pretty']
+        tkwargs['protocol'] = 'rust'
 
         new_target_kwargs = base_target.kwargs.copy()
         # Don't mutate the shallow copied list, instead replace it with a new
@@ -124,40 +155,36 @@ class RustModule(ExtensionModule):
             new_target_kwargs
         )
 
-        e = ExecutableHolder(new_target, self.interpreter)
         test = self.interpreter.make_test(
-            self.interpreter.current_node, [name, e], kwargs)
+            self.interpreter.current_node, (name, new_target), tkwargs)
 
-        return ModuleReturnValue([], [e, test])
+        return ModuleReturnValue(None, [new_target, test])
 
     @noPosargs
-    @permittedKwargs({'input', 'output', 'include_directories', 'c_args', 'args'})
-    def bindgen(self, state: 'ModuleState', args: T.List, kwargs: T.Dict[str, T.Any]) -> ModuleReturnValue:
+    @typed_kwargs(
+        'rust.bindgen',
+        KwargInfo('c_args', ContainerTypeInfo(list, str), default=[], listify=True),
+        KwargInfo('args', ContainerTypeInfo(list, str), default=[], listify=True),
+        KwargInfo('include_directories', ContainerTypeInfo(list, IncludeDirs), default=[], listify=True),
+        KwargInfo(
+            'input',
+            ContainerTypeInfo(list, (File, GeneratedList, BuildTarget, BothLibraries, ExtractedObjects, CustomTargetIndex, CustomTarget, str), allow_empty=False),
+            default=[],
+            listify=True,
+            required=True,
+        ),
+        KwargInfo('output', str, required=True),
+    )
+    def bindgen(self, state: 'ModuleState', args: T.List, kwargs: 'FuncBindgen') -> ModuleReturnValue:
         """Wrapper around bindgen to simplify it's use.
 
         The main thing this simplifies is the use of `include_directory`
         objects, instead of having to pass a plethora of `-I` arguments.
         """
-        header: T.Union[File, CustomTarget, GeneratedList, CustomTargetIndex]
-        _deps: T.Sequence[T.Union[File, CustomTarget, GeneratedList, CustomTargetIndex]]
-        try:
-            header, *_deps = unholder(self.interpreter.source_strings_to_files(listify(kwargs['input'])))
-        except KeyError:
-            raise InvalidArguments('rustmod.bindgen() `input` argument must have at least one element.')
-
-        try:
-            output: str = kwargs['output']
-        except KeyError:
-            raise InvalidArguments('rustmod.bindgen() `output` must be provided')
-        if not isinstance(output, str):
-            raise InvalidArguments('rustmod.bindgen() `output` argument must be a string.')
-
-        include_dirs: T.List[IncludeDirs] = typeslistify(unholder(listify(kwargs.get('include_directories', []))), IncludeDirs)
-        c_args: T.List[str] = stringlistify(listify(kwargs.get('c_args', [])))
-        bind_args: T.List[str] = stringlistify(listify(kwargs.get('args', [])))
+        header, *_deps = self.interpreter.source_strings_to_files(kwargs['input'])
 
         # Split File and Target dependencies to add pass to CustomTarget
-        depends: T.List[BuildTarget] = []
+        depends: T.List['SourceOutputs'] = []
         depend_files: T.List[File] = []
         for d in _deps:
             if isinstance(d, File):
@@ -166,17 +193,18 @@ class RustModule(ExtensionModule):
                 depends.append(d)
 
         inc_strs: T.List[str] = []
-        for i in include_dirs:
+        for i in kwargs['include_directories']:
             # bindgen always uses clang, so it's safe to hardcode -I here
             inc_strs.extend([f'-I{x}' for x in i.to_string_list(state.environment.get_source_dir())])
 
         if self._bindgen_bin is None:
-            # there's some bugs in the interpreter typeing.
-            self._bindgen_bin = T.cast('ExternalProgram', self.interpreter.find_program_impl(['bindgen']).held_object)
+            self._bindgen_bin = state.find_program('bindgen')
 
         name: str
         if isinstance(header, File):
             name = header.fname
+        elif isinstance(header, (BuildTarget, BothLibraries, ExtractedObjects)):
+            raise InterpreterException('bindgen source file must be a C header, not an object or build target')
         else:
             name = header.get_outputs()[0]
 
@@ -186,11 +214,11 @@ class RustModule(ExtensionModule):
             state.subproject,
             {
                 'input': header,
-                'output': output,
+                'output': kwargs['output'],
                 'command': self._bindgen_bin.get_command() + [
                     '@INPUT@', '--output',
                     os.path.join(state.environment.build_dir, '@OUTPUT@')] +
-                    bind_args + ['--'] + c_args + inc_strs +
+                    kwargs['args'] + ['--'] + kwargs['c_args'] + inc_strs +
                     ['-MD', '-MQ', '@INPUT@', '-MF', '@DEPFILE@'],
                 'depfile': '@PLAINNAME@.d',
                 'depends': depends,
@@ -199,8 +227,8 @@ class RustModule(ExtensionModule):
             backend=state.backend,
         )
 
-        return ModuleReturnValue([target], [CustomTargetHolder(target, self.interpreter)])
+        return ModuleReturnValue([target], [target])
 
 
-def initialize(*args: T.List, **kwargs: T.Dict) -> RustModule:
-    return RustModule(*args, **kwargs)  # type: ignore
+def initialize(interp: 'Interpreter') -> RustModule:
+    return RustModule(interp)
